@@ -1,9 +1,9 @@
-/* bignumber.js v1.1.0 https://github.com/MikeMcl/bignumber.js/LICENCE */
+/* bignumber.js v1.3.0 https://github.com/MikeMcl/bignumber.js/LICENCE */
 ;(function ( global ) {
     'use strict';
 
     /*
-      bignumber.js v1.1.0
+      bignumber.js v1.3.0
       A JavaScript library for arbitrary-precision arithmetic.
       https://github.com/MikeMcl/bignumber.js
       Copyright (c) 2012 Michael Mclaughlin <M8ch88l@gmail.com>
@@ -14,7 +14,7 @@
 
     /*
      * The default values below must be integers within the stated ranges (inclusive).
-     * Most of these values can be changed programmatically using BigNumber.config().
+     * Most of these values can be changed during run-time using BigNumber.config().
      */
 
     /*
@@ -73,7 +73,7 @@
     /***********************************************************************************/
 
         P = BigNumber.prototype,
-        DIGITS = '0123456789abcdefghijklmnopqrstuvwxyz',
+        DIGITS = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ$_',
         outOfRange,
         id = 0,
         isValid = /^-?(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?$/i,
@@ -89,10 +89,10 @@
      * Create and return a new instance of a BigNumber object.
      *
      * n {number|string|BigNumber} A numeric value.
-     * [b] {number} The base of n. Integer, 2 to 36 inclusive.
+     * [b] {number} The base of n. Integer, 2 to 64 inclusive.
      */
     function BigNumber( n, b ) {
-        var e, i, isNum, digits, valid,
+        var e, i, isNum, digits, valid, orig,
             x = this;
 
         // Enable constructor usage without new.
@@ -122,6 +122,8 @@
                     n === 0 && 1 / n < 0 ? '-0' : n + ''
         }
 
+        orig = n;
+
         if ( b === e && isValid.test(n) ) {
 
             // Determine sign.
@@ -144,7 +146,7 @@
             if ( b != null ) {
 
                 if ( ( b == (b | 0) || !ERRORS ) &&
-                  !( outOfRange = !( b >= 2 && b <= 36 ) ) ) {
+                  !( outOfRange = !( b >= 2 && b < 65 ) ) ) {
 
                     digits = '[' + DIGITS.slice( 0, b = b | 0 ) + ']+';
 
@@ -154,14 +156,14 @@
 
                     // Any number in exponential form will fail due to the e+/-.
                     if ( valid = new RegExp(
-                      '^' + digits + '(?:\\.' + digits + ')?$', 'i' ).test(n) ) {
+                      '^' + digits + '(?:\\.' + digits + ')?$', b < 37 ? 'i' : '' ).test(n) ) {
 
                         if ( isNum ) {
 
-                            if ( n.replace( '.', '' ).length > 15 ) {
+                            if ( n.replace( /^0\.0*|\./, '' ).length > 15 ) {
 
                                 // 'new BigNumber() number type has more than 15 significant digits: {n}'
-                                ifExceptionsThrow( n, 0 )
+                                ifExceptionsThrow( orig, 0 )
                             }
 
                             // Prevent later check for length on converted number.
@@ -172,7 +174,7 @@
                     } else if ( n != 'Infinity' && n != 'NaN' ) {
 
                         // 'new BigNumber() not a base {b} number: {n}'
-                        ifExceptionsThrow( n, 1, b );
+                        ifExceptionsThrow( orig, 1, b );
                         n = 'NaN'
                     }
                 } else {
@@ -200,7 +202,7 @@
                     if ( n != 'NaN' ) {
 
                         // 'new BigNumber() not a number: {n}'
-                        ifExceptionsThrow( n, 3 )
+                        ifExceptionsThrow( orig, 3 )
                     }
                     x['s'] = null
                 }
@@ -216,7 +218,7 @@
         }
 
         // Exponential form?
-        if ( ( i = n.search(/e/i) ) > 0 ) {
+        if ( ( i = n.search( /e/i ) ) > 0 ) {
 
             // Determine exponent.
             if ( e < 0 ) {
@@ -231,16 +233,19 @@
             e = n.length
         }
 
-        // Disallow numbers over 15 digits if number type.
-        if ( b = n.length, isNum && b > 15 ) {
+        // Determine leading zeros.
+        for ( i = 0; n.charAt(i) == '0'; i++ ) {
+        }
+
+        b = n.length;
+
+        // Disallow numbers with over 15 significant digits if number type.
+        if ( isNum && b > 15 && n.slice(i).length > 15 ) {
 
             // 'new BigNumber() number type has more than 15 significant digits: {n}'
-            ifExceptionsThrow( n, 0 )
-        }
-
-        // Determine leading zeros.
-        for ( i = id = 0; n.charAt(i) == '0'; i++ ) {
+            ifExceptionsThrow( orig, 0 )
         }
+        id = 0;
 
         // Overflow?
         if ( ( e -= i + 1 ) > MAX_EXP ) {
@@ -404,32 +409,33 @@
 
     // Assemble error messages. Throw BigNumber Errors.
     function ifExceptionsThrow( arg, i, j, isArray, isRange, isErrors) {
+
         if ( ERRORS ) {
-            var method = ['new BigNumber', 'cmp', 'div', 'eq', 'gt', 'gte', 'lt',
+            var error,
+                method = ['new BigNumber', 'cmp', 'div', 'eq', 'gt', 'gte', 'lt',
                      'lte', 'minus', 'mod', 'plus', 'times', 'toFr'
                     ][ id ? id < 0 ? -id : id : 1 / id < 0 ? 1 : 0 ] + '()',
-                error = outOfRange ? ' out of range' : ' not a' +
+                message = outOfRange ? ' out of range' : ' not a' +
                   ( isRange ? ' non-zero' : 'n' ) + ' integer';
 
-            error = ( [
+            message = ( [
                 method + ' number type has more than 15 significant digits',
                 method + ' not a base ' + j + ' number',
-                method + ' base' + error,
+                method + ' base' + message,
                 method + ' not a number' ][i] ||
                   j + '() ' + i + ( isErrors
                     ? ' not a boolean or binary digit'
-                    : error + ( isArray
+                    : message + ( isArray
                       ? ' or not [' + ( outOfRange
                         ? ' negative, positive'
                         : ' integer, integer' ) + ' ]'
                       : '' ) ) ) + ': ' + arg;
 
             outOfRange = id = 0;
-            throw {
-                name : 'BigNumber Error',
-                message : error,
-                toString : function () {return this.name + ': ' + this.message}
-            }
+            error = new Error(message);
+            error['name'] = 'BigNumber Error';
+
+            throw error
         }
     }
 
@@ -486,7 +492,9 @@
             return str
         }
 
-        nStr = nStr.toLowerCase();
+        if ( baseIn < 37 ) {
+            nStr = nStr.toLowerCase()
+        }
 
         /*
          * If non-integer convert integer part and fraction part separately.
@@ -1101,7 +1109,8 @@
                     ? x
 
                     // Both are zero.
-                    : 0 )
+                    // IEEE 754 (2008) 6.3: n - n = -0 when rounding to -Infinity
+                    : ROUNDING_MODE == 3 ? -0 : 0 )
             }
         }
 
@@ -1172,7 +1181,15 @@
         // Underflow?
         if ( ye < MIN_EXP || !xc[0] ) {
 
-            // Result must be zero.
+            /*
+             * Following IEEE 754 (2008) 6.3,
+             * n - n = +0  but  n - n = -0 when rounding towards -Infinity.
+             */
+            if ( !xc[0] ) {
+                y['s'] = ROUNDING_MODE == 3 ? -1 : 1
+            }
+
+            // Result is zero.
             xc = [ye = 0]
         }
 
@@ -1290,9 +1307,9 @@
                 // y is non-zero?
                 return yc[0]
                   ? y
-                  : new BigNumber( xc[0]
 
-                    // x is non-zero?
+                  // x is non-zero?
+                  : new BigNumber( xc[0]
                     ? x
 
                     // Both are zero. Return zero.
@@ -1445,50 +1462,125 @@
      * this BigNumber, rounded according to DECIMAL_PLACES and ROUNDING_MODE.
      */
     P['squareRoot'] = P['sqrt'] = function () {
-        var estimate, r, approx,
+        var i, n, r, re, t,
             x = this,
-            xc = x['c'],
-            i = x['s'],
+            c = x['c'],
+            s = x['s'],
             e = x['e'],
+            dp = DECIMAL_PLACES,
+            rm = ROUNDING_MODE,
             half = new BigNumber('0.5');
 
         // Negative/NaN/Infinity/zero?
-        if ( i !== 1 || !xc || !xc[0] ) {
-            return new BigNumber( !i || i < 0 && ( !xc || xc[0] )
+        if ( s !== 1 || !c || !c[0] ) {
+
+            return new BigNumber( !s || s < 0 && ( !c || c[0] )
               ? NaN
-              : xc ? x : 1 / 0 )
+              : c ? x : 1 / 0 )
         }
 
-        // Estimate.
-        i = Math.sqrt( x['toS']() );
+        // Initial estimate.
+        s = Math.sqrt( x['toS']() );
+        ROUNDING_MODE = 1;
 
-        // Math.sqrt underflow/overflow?
-        // Pass x to Math.sqrt as integer, then adjust the exponent of the result.
-        if ( i == 0 || i == 1 / 0 ) {
-            estimate = xc.join('');
+        /*
+          Math.sqrt underflow/overflow?
+          Pass x to Math.sqrt as integer, then adjust the exponent of the result.
+         */
+        if ( s == 0 || s == 1 / 0 ) {
+            n = c.join('');
 
-            if ( !( estimate.length + e & 1 ) ) {
-                estimate += '0'
+            if ( !( n.length + e & 1 ) ) {
+                n += '0'
             }
+            r = new BigNumber( Math.sqrt(n) + '' );
 
-            r = new BigNumber( Math.sqrt(estimate).toString() );
+            // r may still not be finite.
+            if ( !r['c'] ) {
+                r['c'] = [1]
+            }
             r['e'] = ( ( ( e + 1 ) / 2 ) | 0 ) - ( e < 0 || e & 1 )
         } else {
-            r = new BigNumber( i.toString() )
+            r = new BigNumber( n = s.toString() )
         }
+        re = r['e'];
+        s = re + ( DECIMAL_PLACES += 4 );
 
-        i = r['e'] + ( DECIMAL_PLACES += 4 );
+        if ( s < 3 ) {
+            s = 0
+        }
+        e = s;
 
-        // Newton-Raphson loop.
-        do {
-            approx = r;
-            r = half['times']( approx['plus']( x['div'](approx) ) )
-        } while ( approx['c'].slice( 0, i ).join('') !==
-                       r['c'].slice( 0, i ).join('') );
+        // Newton-Raphson iteration.
+        for ( ; ; ) {
+            t = r;
+            r = half['times']( t['plus']( x['div'](t) ) );
 
-        rnd( r, DECIMAL_PLACES -= 4, 10 );
+            if ( t['c'].slice( 0, s ).join('') === r['c'].slice( 0, s ).join('') ) {
+                c = r['c'];
 
-        return r
+                /*
+                  The exponent of r may here be one less than the final result
+                  exponent (re), e.g 0.0009999 (e-4) --> 0.001 (e-3), so adjust
+                  s so the rounding digits are indexed correctly.
+                 */
+                s = s - ( n && r['e'] < re );
+
+                /*
+                  The 4th rounding digit may be in error by -1 so if the 4 rounding
+                  digits are 9999 or 4999 (i.e. approaching a rounding boundary)
+                  continue the iteration.
+                 */
+                if ( c[s] == 9 && c[s - 1] == 9 && c[s - 2] == 9 &&
+                        ( c[s - 3] == 9 || n && c[s - 3] == 4 ) ) {
+
+                    /*
+                      If 9999 on first run through, check to see if rounding up
+                      gives the exact result as the nines may infinitely repeat.
+                     */
+                    if ( n && c[s - 3] == 9 ) {
+                        t = r['round']( dp, 0 );
+
+                        if ( t['times'](t)['eq'](x) ) {
+                            ROUNDING_MODE = rm;
+                            DECIMAL_PLACES = dp;
+
+                            return t
+                        }
+                    }
+                    DECIMAL_PLACES += 4;
+                    s += 4;
+                    n = ''
+                } else {
+
+                    /*
+                      If the rounding digits are null, 0000 or 5000, check for an
+                      exact result. If not, then there are further digits so
+                      increment the 1st rounding digit to ensure correct rounding.
+                     */
+                    if ( !c[e] && !c[e - 1] && !c[e - 2] &&
+                            ( !c[e - 3] || c[e - 3] == 5 ) ) {
+
+                        // Truncate to the first rounding digit.
+                        if ( c.length > e - 2 ) {
+                            c.length = e - 2
+                        }
+
+                        if ( !r['times'](r)['eq'](x) ) {
+
+                            while ( c.length < e - 3 ) {
+                                c.push(0)
+                            }
+                            c[e - 3]++
+                        }
+                    }
+                    ROUNDING_MODE = rm;
+                    rnd( r, DECIMAL_PLACES = dp, 10 );
+
+                    return r
+                }
+            }
+        }
     };
 
 
@@ -1800,7 +1892,7 @@
      * that is equal to or greater than TO_EXP_POS, or a negative exponent equal
      * to or less than TO_EXP_NEG, return exponential notation.
      *
-     * [b] {number} Integer, 2 to 36 inclusive.
+     * [b] {number} Integer, 2 to 64 inclusive.
      */
     P['toString'] = P['toS'] = function ( b ) {
         var u, str, strL,
@@ -1850,7 +1942,7 @@
 
             if ( b != null ) {
 
-                if ( !( outOfRange = !( b >= 2 && b <= 36 ) ) &&
+                if ( !( outOfRange = !( b >= 2 && b < 65 ) ) &&
                   ( b == (b | 0) || !ERRORS ) ) {
                     str = convert( str, b | 0, 10, x['s'] );
 
